package de.uni_goettingen.med.mi.bdtparser.core.bdt_structur;

/**
 * Class used to store a line from a bdt file. The line is stored split up in length, field number and data.
 * The class also removes the CRLF at the end of the data part.
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BdtLine {
    /**
     * The beginning of the length part
     */
    public final static int LINELENGTHBEGINN = 0;
    /**
     * the beginning of the fieldnumber
     */
    public final static int LINEFIELDNUMBERBEGINN = 3;
    /**
     * the beginning of the data
     */
    public final static int LINEFIELDDATABEGINN = 7;
    private String length;
    private String fieldNumber;
    private String data;

    /**
     * creates an instance from a given line
     *
     * @param line the line that should be stored
     */
    public BdtLine(String line) {
        setLength(line.substring(LINELENGTHBEGINN, LINEFIELDNUMBERBEGINN));
        setFieldNumber(line.substring(LINEFIELDNUMBERBEGINN, LINEFIELDDATABEGINN));
        setData(line.substring(LINEFIELDDATABEGINN));
    }

    /**
     * Getter fo the length
     *
     * @return the lenght
     */
    public String getLength() {
        return length;
    }

    /**
     * Setter for the length
     *
     * @param length the length
     */
    public void setLength(String length) {
        this.length = length;
    }

    /**
     * Getter for the FieldCharacteristic
     *
     * @return the fieldnumber
     */
    public String getFieldNumber() {
        return fieldNumber;
    }

    /**
     * setter for the fieldnumber
     *
     * @param fieldNumber
     */
    public void setFieldNumber(String fieldNumber) {
        this.fieldNumber = fieldNumber;
    }

    /**
     * Getter fo the Datapart
     *
     * @return the datapart of the bdtline
     */
    public String getData() {
        return data;
    }

    /**
     * Setter fo the data part
     * removes the CRLF at the end
     *
     * @param data the data part of the BDT line
     */
    public void setData(String data) {
        if (!data.isEmpty()) {
            for (int i = 0; i < 2; i++) {
                if (data.charAt(data.length() - 1) == '\r' || data.charAt(data.length() - 1) == '\n') {
                    data = data.substring(0, data.length() - 1);
                }
            }
        }
        this.data = data;
    }

}