package de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field;

import java.util.concurrent.atomic.AtomicLong;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.BdtLine;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.Sentence;

/**
 * This class is used to store a Field (or a line) of a bdt file
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class Field {

    public final Long ID;
    public boolean used;
    private static AtomicLong idCounter = new AtomicLong();
    private String length;
    private String number;
    private String data;
    private Sentence sentence;
    private boolean valid;

    /**
     * Returns false if a field was declared invalid while validating
     *
     * @return true if the field is valid, false otherwise
     */
    public boolean isValid() {
        return valid;
    }

    /**
     * Sets the flag if the field is valid or not
     *
     * @param valid the flag
     */
    public void setValid(boolean valid) {
        this.valid = valid;
    }

    /**
     * This Methode returns the field in its BDT-format, but without CRLF at the end
     *
     * @return the field as String
     */
    public String toString() {
        return length + number + data;
    }

    /**
     * Getter for the length
     *
     * @return the length
     */
    public String getLength() {
        return length;
    }

    /**
     * setter for the length
     *
     * @param length the length
     */
    public void setLength(String length) {
        if (length.length() == 1)
            this.length = "00" + length;
        if (length.length() == 2)
            this.length = "0" + length;
        this.length = length;
    }

    /**
     * getter for the fieldnumber
     *
     * @return the fieldnumer
     */
    public String getNumber() {
        return number;
    }

    /**
     * setter for the fieldnumer
     *
     * @param number the fieldnumer
     */
    public void setNumber(long number) {
        if (number < 100)
            setNumber("00" + number);
        else if (number < 1000)
            setNumber("0" + number);
        else
            setNumber(String.valueOf(number));
    }

    /**
     * setter fo the field number
     *
     * @param number the number to be set
     */
    public void setNumber(String number) {
        this.number = number;
        if (!FieldTable.fieldTable.containsKey(Integer.parseInt(number))) {
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append("Unknown fieldnumber: ");
            stringBuilder.append(number);
        }
    }

    /**
     * getter for teh Data
     *
     * @return the data
     */
    public String getData() {
        return data;
    }

    /**
     * setter for the Data
     *
     * @param data the Data
     */
    public void setData(String data) {
        this.data = data;
    }

    /**
     * getter for the sentence
     *
     * @return a reference to the sentece
     */
    public Sentence getSentence() {
        return sentence;
    }

    /**
     * setter for the sentence
     *
     * @param sentence the belonging sentence
     */
    public void setSentence(Sentence sentence) {
        this.sentence = sentence;
    }

    /**
     * Constructor using a bdtline to create the field
     *
     * @param bdtLine the bdt line
     * @param sentence the sentence the field is belonging to
     */
    public Field(BdtLine bdtLine, Sentence sentence) {
        this(bdtLine.getLength(), bdtLine.getFieldNumber(), bdtLine.getData(), sentence);

    }

    /**
     * Constructor for a field
     *
     * @param length the length
     * @param number the fieldnumber
     * @param data the data
     * @param sentence the sentence the field is belonging to
     */
    public Field(String length, String number, String data, Sentence sentence) {
        setValid(true);
        setSentence(sentence);
        setLength(length);
        setNumber(number);
        setData(data);
        ID = idCounter.getAndIncrement();

    }

}
