package de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field;

import java.util.TreeMap;

/**
 * This class is used to store the characteristic of a Field used in the BDT-Data format.
 * The class allows to access the field characteristics by their bdt field number.
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class FieldTable {

    /**
     * the Treemap used to store the known Field caracteristics
     */
    final static TreeMap<Integer, FieldCharacteristic> fieldTable = new TreeMap<Integer, FieldCharacteristic>();

    static {
        for (FieldCharacteristic fn : FieldCharacteristic.values()) {
            fieldTable.put(Integer.parseInt(fn.getNumber()), fn);
        }
    }

    /**
     * Delivers the characteristic of a field according to the given fieldnumber
     *
     * @param fieldnumber the fieldnumber of the field
     * @return the characteristic of the field if known , null ottherwise
     */
    public static FieldCharacteristic getCharateristic(int fieldnumber) {
        try {
            return FieldCharacteristic.valueOf(fieldTable.get(fieldnumber).getName());
        } catch (NullPointerException npe) {
            return null;
        }
    }

}
