package de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence;

import java.util.TreeMap;

/**
 * The Class used to store the table of all known references
 * use it to reference the SentenceCharacteristics by their number
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class SentenceTable {

    /**
     * the table, will be initialised in a static block, and shouldn't not modified while the program is running
     */
    final private static TreeMap<Integer, SentenceCharacteristic> SentenceTable = new TreeMap<>();

    static {
        for (SentenceCharacteristic sentence : SentenceCharacteristic.values()) {
            SentenceTable.put(Integer.parseInt(sentence.id), sentence);
        }
    }

    /**
     * The method to reference the characteristics by their numbers
     *
     * @param sentencenumber the number of the sentence that should be referenced
     * @return the SentenceCharacteristic, could be null
     */
    public static SentenceCharacteristic getCharateristic(int sentencenumber) {
        try {
            return SentenceCharacteristic.valueOf(SentenceTable.get(sentencenumber).name());
        } catch (NullPointerException npe) {
            return null;
        }
    }

}
