package de.uni_goettingen.med.mi.bdtparser.core.validator;

/**
 * Exception thrown by the {@link Validator}<br/>
 * The Exception holds the errorCode (see {@link Validator}for more) a boolean that is true if the error was critical.
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class ValidationException extends Exception {
    private boolean critical;
    private int errorCode;

    /**
     * Constructor for the Exceptions, needs the error message, errorCode and a boolean if the error was critical
     *
     * @param message the error message as String
     * @param critical true if at least one error was critical
     * @param errorCode the errorCode (see {@link Validator}for more)
     */
    ValidationException(String message, boolean critical, int errorCode) {
        super(message);
        this.critical = critical;
        this.errorCode = errorCode;
    }

    /**
     * Getter for the errorCode
     *
     * @return the errorCode
     */
    public int getErrorCode() {
        return errorCode;
    }

    /**
     * getter for Critical
     *
     * @return if the error was critical
     */
    public boolean isCritical() {
        return critical;
    }
}
