package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

/**
*
* @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
*/
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Class used to store an address including the reference to fields from the BDT-files
 * Variables ort und street are named in german as the specific field of the bdt files
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BDTAdresse extends Referencer {
    private BDTValue<String> ort;
    private BDTValue<String> strasse;
    private String plz;

    /**
     * Default Constructor
     */
    public BDTAdresse() {
    }

    /**
     * Constructor wich stored the given street and and the city
     * 
     * @param strasse the street part of the address
     * @param ort the city part
     */
    public BDTAdresse(BDTValue<String> strasse, BDTValue<String> ort) {
        this.strasse = strasse;
        this.ort = ort;
    }

    /**
     * getter for String value of the street part of the address
     * 
     * @return
     */
    public String getStreet() {
        if (strasse == null)
            throw new IllegalStateException("The street value isn't set.");
        return strasse.toString();
    }

    /**
     * getter for the String value of the city part of the address
     * 
     * @return
     */
    public String getCity() {
        String city = "";
        if (ort == null)
            throw new IllegalStateException("The city value isn't set.");
        city = ort.toString();
        if (getZipCode() != null) {
            city = city.replace(getZipCode(), ""); // remove zipcode from ort field
            city = city.trim(); // remove leading and trailing spaces
        } else {
            System.out.println("ERROR getCity() " + city + " " + ort + " " + getZipCode());
        }

        return city;
    }

	public String getZipCode() {
		if (plz != null && !plz.isEmpty()) {
			return plz;
		} else {
			if (ort == null || ort.getValue() == null)
				return null;
			String zipCode;
			try {
				Pattern pattern = Pattern.compile("[0-9]{5}");
				Matcher matcher = pattern.matcher(ort.getValue());
				if (matcher.find()) {
					zipCode = matcher.group();
					return zipCode;
				}
				return null;
			} catch (Exception e) {
				return null;
			}
		}
	}

    /**
     * getter for the BDT-Value from the Street
     * 
     * @return the street part
     */
    public BDTValue<String> getStrasse() {
        return strasse;
    }

    /**
     * setter for the street-part as BDTValue
     * 
     * @param strasse the street part
     */
    public void setStrasse(BDTValue<String> strasse) {
        this.strasse = strasse;
    }

    /**
     * getter for the BDT-Value of the street
     * 
     * @return the city part
     */
    public BDTValue<String> getOrt() {
        return ort;
    }

    /**
     * setter for the BDTValue of the city
     * 
     * @param ort the city
     */
    public void setOrt(BDTValue<String> ort) {
        this.ort = ort;
    }

    public void setZipCode(String zipCode) {
        if (ort != null && ort.getValue() != null)
            if (ort.getValue().contains(zipCode))
                ort.setValue(ort.getValue().replaceAll(getZipCode(), zipCode));
            else
                ort.setValue(zipCode);
    }
    /**
     * setter for the BDTValue of the zip code
     *
     * @param zipCode the zip code of the city
     */
	public void setZipCode(BDTValue<String> zipCode) {
		if (zipCode.getValue() != null) {
				this.plz = zipCode.getValue();
		}
	}

}
