package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.util.LinkedList;

/**
 * Class used to store the attributes of a doctor extracted from the BDT-Files.
 * Its also includes the references to fields where the attributes are stored.
 * Extends the referencer class
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BDTArzt extends Referencer {
    private BDTValue<String> name;
    private BDTValue<String> arztgruppe;

    /**
     * setter for BDTValue Name
     * 
     * @param name the name of the doctor
     */
    public void setName(BDTValue<String> name) {
        this.name = name;
    }

    /**
     * the setter for the BDTValue of the specialisation group of the doctor
     * 
     * @param arztgruppe the specialisation group of the doctor
     */
    public void setArztgruppe(BDTValue<String> arztgruppe) {
        this.arztgruppe = arztgruppe;
    }

    /**
     * Default Constructor for the doctor
     * 
     * @param name the name of the doctor
     */
    public BDTArzt(BDTValue<String> name) {
        this.name = name;
    }

    /**
     * Getter for the name BDTValue
     * 
     * @return the name
     */
    public BDTValue<String> getName() {
        return name;
    }

    /**
     * getter for the specialisation group
     * 
     * @return the specialisation group
     */
    public BDTValue<String> getArztgruppe() {
        return arztgruppe;
    }

    /**
     * Overwrites the getAsReferenceables methode from the referencer class
     * 
     * @return a list of all referencable Values in this class
     * @throws IllegalAccessException never
     */
    @Override
    public LinkedList getAsReferenceables() throws IllegalAccessException {
        LinkedList<BDTValue<String>> list = new LinkedList<>();
        list.add(name);
        list.add(arztgruppe);
        return list;
    }
}
