package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.LinkedList;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * Class to used to store all attributes of a doctors surgery including a HashMap all patients
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BDTArztpraxis extends Referencer {
    public final static Logger logger = LogManager.getLogger(BDTArztpraxis.class);
    private HashMap<Long, BDTPatient> patients;
    protected BDTValue<Integer> typ;
    protected BDTValue<Integer> anzahlAerzte;
    protected BDTValue<String> ortPLZ;
    protected BDTValue<Integer> praxisnummer;
    protected BDTValue<Integer> arztNummer;
    protected LinkedList<BDTArzt> aerzte;
    protected BDTValue<String> KBV_Puefnummer;
    protected BDTValue<String> softwareverantworlicher;
    protected BDTValue<String> software;
    protected BDTValue<String> hardware;
    protected BDTValue<String> strasse;
    protected LinkedList<BDTValue<String>> arzteMitLeistungskennzeichen;
    protected BDTValue<String> telefon;
    protected BDTValue<String> telefax;
    protected BDTValue<String> modemnummer;

    /**
     * returns the HashMMap of the patitients
     * 
     * @return the hashmap
     */
    public HashMap<Long, BDTPatient> getPatients() {
        return patients;
    }

    /**
     * setter for the patient Hashmap
     * 
     * @param patients the hashmap
     */
    public void setPatients(HashMap<Long, BDTPatient> patients) {
        this.patients = patients;
    }

    /**
     * public default constructor
     */
    public BDTArztpraxis() {
        patients = new HashMap<>();
        aerzte = new LinkedList<>();
        arzteMitLeistungskennzeichen = new LinkedList<>();
    }

    /**
     * Overwrites the method from the super class.
     * returns all attributes as a list of references
     * 
     * @return the list of references
     */
    public LinkedList<Referenceable> getAsReferenceables() {
        LinkedList<Referenceable> list = new LinkedList<>();
        try {
            Field[] fields = this.getClass().getDeclaredFields();
            Referencer.addFields(fields, list, this);
            for (BDTArzt arzt : aerzte)
                list.addAll(arzt.getAsReferenceables());
            list.addAll(arzteMitLeistungskennzeichen);
        } catch (IllegalAccessException iae) {
            logger.error("cant get all Data from arztprxis as referencableList" + iae.getMessage());
        }
        return list;
    }

    /**
     * attaches all attributes after each one , and returns them as a string
     * 
     * @return the string representation of this class
     */
    @Override
    public String toString() {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("KBV_Puefnummer: ").append(KBV_Puefnummer).append(" Softwareverantwortlicher: ").append(softwareverantworlicher).append(" Software: ").append(software)
                .append(" Arztnummer: ").append(arztNummer).append(" Praxistyp: ").append(typ);
        return stringBuilder.toString();
    }

    /**
     * adds an doctor to the list of doctors working in the surgery
     * 
     * @param arzt the doctor to add
     */
    public void addArzt(BDTArzt arzt) {
        if (aerzte == null)
            aerzte = new LinkedList<>();
        aerzte.add(arzt);
    }

    // getter and setter
    public BDTValue<String> getKBV_Puefnummer() {
        return KBV_Puefnummer;
    }

    public void setKBV_Puefnummer(BDTValue<String> KBV_Puefnummer) {
        this.KBV_Puefnummer = KBV_Puefnummer;
    }

    public BDTValue<String> getSoftwareverantworlicher() {
        return softwareverantworlicher;
    }

    public void setSoftwareverantworlicher(BDTValue<String> softwareverantworlicher) {
        this.softwareverantworlicher = softwareverantworlicher;
    }

    public BDTValue<String> getSoftware() {
        return software;
    }

    public void setSoftware(BDTValue<String> software) {
        this.software = software;
    }

    public BDTValue<String> getHardware() {
        return hardware;
    }

    public void setHardware(BDTValue<String> hardware) {
        this.hardware = hardware;
    }

    public BDTValue<String> getStrasse() {
        return strasse;
    }

    public void setStrasse(BDTValue<String> strasse) {
        this.strasse = strasse;
    }

    public LinkedList<BDTValue<String>> getArzteMitLeistungskennzeichen() {
        return arzteMitLeistungskennzeichen;
    }

    public void setArzteMitLeistungskennzeichen(LinkedList<BDTValue<String>> arzteMitLeistungskennzeichen) {
        this.arzteMitLeistungskennzeichen = arzteMitLeistungskennzeichen;
    }

    public BDTValue<String> getTelefon() {
        return telefon;
    }

    public void setTelefon(BDTValue<String> telefon) {
        this.telefon = telefon;
    }

    public BDTValue<String> getTelefax() {
        return telefax;
    }

    public void setTelefax(BDTValue<String> telefax) {
        this.telefax = telefax;
    }

    public BDTValue<String> getModemnummer() {
        return modemnummer;
    }

    public void setModemnummer(BDTValue<String> modemnummer) {
        this.modemnummer = modemnummer;
    }

    public BDTValue<Integer> getArztNummer() {
        return arztNummer;
    }

    public void setArztNummer(BDTValue<Integer> arztNummer) {
        this.arztNummer = arztNummer;
    }

    public void setTyp(BDTValue<Integer> typ) {
        this.typ = typ;
    }

    public void setAnzahlAerzte(BDTValue<Integer> anzahlAerzte) {
        this.anzahlAerzte = anzahlAerzte;
    }

    public void setOrtPLZ(BDTValue<String> ortPLZ) {
        this.ortPLZ = ortPLZ;
    }

    public void setPraxisnummer(BDTValue<Integer> praxisnummer) {
        this.praxisnummer = praxisnummer;
    }

    public void setAerzte(LinkedList<BDTArzt> aerzte) {
        this.aerzte = aerzte;
    }

    public BDTValue<Integer> getTyp() {
        return typ;
    }

    public BDTValue<Integer> getAnzahlAerzte() {
        return anzahlAerzte;
    }

    public BDTValue<String> getOrtPLZ() {
        return ortPLZ;
    }

    public BDTValue<Integer> getPraxisnummer() {
        return praxisnummer;
    }

    public LinkedList<BDTArzt> getAerzte() {
        return aerzte;
    }
}
