package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.util.LinkedList;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;

/**
 * Class to store a report and the test including it.
 * 
 * @author Department of Medical Informatics of the University Medical Center Goettingen (Hügel/Pung)
 */
public class BDTBefundArt extends ReferencableReferencer {
	private String befundart;
	protected LinkedList<BDTTest> tests;

	/**
	 * Constructor given the result type and a reference to the field where it was stored
	 * 
	 * @param befundart type of report
	 * @param reference reference to field
	 */
	public BDTBefundArt(String befundart, Field reference) {
		super(reference);
		this.befundart = befundart;
		tests = new LinkedList<>();
	}

	/**
	 * returns all stored values as List of references
	 * 
	 * @return the list of references
	 * @throws IllegalAccessException if a field is inaccessible
	 */
	@SuppressWarnings("rawtypes")
	@Override
	public LinkedList<Referenceable> getAsReferenceables() throws IllegalAccessException {
		LinkedList<Referenceable> list = super.getAsReferenceables();
		for (BDTTest test : tests)
			list.addAll(test.getAsReferenceables());
		return list;
	}

	/**
	 * Getter for the type of record
	 * 
	 * @return the type of record
	 */
	@Override
	public String getValue() {
		return befundart;
	}

	/**
	 * setter for type of record
	 * 
	 * @param value the value of type of record
	 */
	@Override
	public void setValue(String value) {
		befundart = value;
	}

	/**
	 * Getter to retrieve a list of all tests
	 *
	 * @return
	 */
	public LinkedList<BDTTest> getTests() {
		return tests;
	}

}	