package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;

/**
 * Class to store BDTDiagnose from the BDT-files either as ICD code or as text
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BDTDiagnose extends ReferencableReferencer {
    private String description;
    private boolean ICD;
    private boolean billingDiagnosis;

    /**
     * Constructor for diagonse
     * 
     * @param description the description or the icd code of the diagnosis
     * @param reference the bdt-field which stores the value
     * @param isICD true if descriptions holds an icd code
     * @param billingDiagnosis true if descriptions holds an billing diagnosis
     */
    public BDTDiagnose(String description, Field reference, boolean isICD, boolean billingDiagnosis) {
        super(reference);
        this.description = description;
        this.ICD = isICD;
        this.billingDiagnosis = billingDiagnosis;
    }

    /**
     * Is the stored value a ICD coded diagnosis
     * 
     * @return true if it is, else otherwise
     */
    public boolean isICD() {
        return ICD;
    }

    /**
     * set if the diagnosis is a icd;
     * 
     * @param ICD true if it is, else otherwise
     */
    public void setICD(boolean ICD) {
        this.ICD = ICD;
    }

    /**
     * Is the stored value a billing diagnosis
     * 
     * @return true if it is, else otherwise
     */
    public boolean isBillingDiagnosis() {
        return billingDiagnosis;
    }

    /**
     * set if the diagnosis is billing diagnosis;
     * 
     * @param billingDiagnosis true if it is, else otherwise
     */
    public void setBillingDiagnosis(boolean billingDiagnosis) {
        this.billingDiagnosis = billingDiagnosis;
    }

    /**
     * Overwrites the methode from the super class
     * getter for the description of the diagnosis
     * 
     * @return the description of the diagnosis
     */
    @Override
    public String getValue() {
        return description;
    }

    /**
     * Overwrites the method from the super class
     * the setter for the description of the diagnosis
     * 
     * @param value the value
     */
    @Override
    public void setValue(String value) {
        description = value;

    }

}
