package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.datapackage.Datapackage;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;
import de.uni_goettingen.med.mi.bdtparser.core.io.writer.Unmarshaller;

import java.util.HashMap;
import java.util.LinkedList;

public class BDTFileSaverViewer extends BDTFileSaver{

    /**
     * Constructor
     *
     * @param datapackage the datapackage used to reconstruct the bdt files
     * @param path
     */
    public BDTFileSaverViewer(Datapackage datapackage, String path) {
        super(datapackage, path);
    }

    /**
     * Saves the data from the vpr back to bdt files using the old datapackage structure and removes invalid and not in the vpr stored fields
     *
     * @param arztpraxis the arxtpraxis from which the data should be written back
     */
    @Override
    public void save(BDTArztpraxis arztpraxis) {
        HashMap<Long, BDTPatient> patients = arztpraxis.getPatients();
        LinkedList<Referenceable> mergedLists = arztpraxis.getAsReferenceables();
        for (BDTPatient patient : patients.values()) {
            LinkedList<Referenceable> patientAsReferenceable = patient.getAsReferenceables();
            mergedLists.addAll(patientAsReferenceable);
        }
        for (Referenceable referencable : mergedLists) {
            if (referencable != null && referencable.getValue() != null) {
                for (Object referenceObject : referencable.getReferences()) {
                    Field reference = ((Field) referenceObject);
                    if(reference.getNumber().equals("3000")) {
                        setField("temp_" + reference.getData(), reference);
                    } else{
                        setField(referencable.getValue().toString(), reference);
                    }
                }
            }
        }
        markDefaultSentences();
        reduceToSpecifiedFields();
        reduceDatapackage();
        updateSize();
        Unmarshaller unmarshaller = new Unmarshaller(getDatapackage(),getPath());
        unmarshaller.run();
    }
}
