package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.util.LinkedList;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;

/**
 * Class used to store all the attributes of a medical test during an examination from the bdtfiles.
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BDTTest extends ReferencableReferencer {
    private String testIdent;
    protected BDTValue<String> testBezeichnung;
    protected BDTValue<String> ergebnis;
    protected BDTValue<String> einheit;

    /**
     * Constructor given the identification of the the test and a reference to the field where it was stored
     * 
     * @param testIdent the identification of the test
     * @param reference the reference to field
     */
    public BDTTest(String testIdent, Field reference) {
        super(reference);
        this.testIdent = testIdent;
    }

    /**
     * returns a list holding all the values of this class as referenceable
     * 
     * @return the list of all vlues
     * @throws IllegalAccessException if a Field is not accessible
     */
    @SuppressWarnings("rawtypes")
    @Override
    public LinkedList<Referenceable> getAsReferenceables() throws IllegalAccessException {
        LinkedList<Referenceable> list = new LinkedList<>();
        list.add(this);
        list.add(testBezeichnung);
        list.add(ergebnis);
        list.add(einheit);
        list.addAll(super.getAsReferenceables());
        return list;
    }

    /**
     * Returns the identification from the test
     * 
     * @return the identification
     */
    @Override
    public String getValue() {
        return testIdent;
    }

    /**
     * setter for the test identification
     * 
     * @param value the value
     */
    @Override
    public void setValue(String value) {
        testIdent = value;
    }

    // getter and setter for the stored values

    public BDTValue<String> getTestBezeichnung() {
        return testBezeichnung;
    }

    public void setTestBezeichnung(BDTValue<String> testBezeichnung) {
        this.testBezeichnung = testBezeichnung;
    }

    public BDTValue<String> getErgebnis() {
        return ergebnis;
    }

    public void setErgebnis(BDTValue<String> ergebnis) {
        this.ergebnis = ergebnis;
    }

    public BDTValue<String> getEinheit() {
        return einheit;
    }

    public void setEinheit(BDTValue<String> einheit) {
        this.einheit = einheit;
    }

}