package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.util.LinkedList;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;

/**
 * class used to store a value in the VPR also contains a Reference to the original {@link de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field} to allow to rewrite the BDT-Files
 * Implements the Referenceable Interface
 * 
 * @param <E> the typ of the stored Value, E has to have a valid toString method
 *
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class BDTValue<E> implements Referenceable<E> {
    protected E value;
    protected LinkedList<Field> references;

    /**
     * public default constructor
     * 
     * @param value the value stored in the bdt-file
     * @param reference the field in which the value is stored
     */
    public BDTValue(E value, Field reference) {
        this.value = value;
        this.references = new LinkedList<>();
        this.references.add(reference);
    }

    /**
     * adds a single Reference to the list the of the references
     * Used to store all the fields in which the value part is stored.
     * 
     * @param reference
     */
    @Override
    public void addReference(Field reference) {
        if (!references.contains(reference))
            references.add(reference);
    }

    /**
     * Returns the reference to the original Field in the Sentence Structure. Allows to rewrites the BDT-files.
     * 
     * @return the reference to the original field
     */
    @Override
    public LinkedList<Field> getReferences() {
        return references;
    }

    /**
     * Returns the Value of the field
     * 
     * @return the value
     */
    @Override
    public E getValue() {
        return value;
    }

    /**
     * setter for the value of the field
     * 
     * @param value
     */
    @Override
    public void setValue(E value) {
        this.value = value;
    }

    /**
     * returns the String representation of the value part
     * 
     * @return
     */
    @Override
    public String toString() {
        if (value == null)
            return "";
        return value.toString();
    }
}
