package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.util.LinkedList;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;

/**
 * An abstract class which extends the Referencer class and implements the referencable Interface. This means the class export a methode to get all its referencable fields,
 * but it's also referenceable.
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public abstract class ReferencableReferencer extends Referencer implements Referenceable<String> {
    protected LinkedList<Field> references;

    /**
     * Protected Constructor which adds a given reference to the list of references
     * 
     * @param reference the reference of the value stored in the class
     */
    protected ReferencableReferencer(Field reference) {
        references = new LinkedList<>();
        if (references != null)
            references.add(reference);
    }

    /**
     * Overwrites the addReference method from the referencable Interface.
     * adds a reference to the list of references
     * 
     * @param reference the reference to a field which should be added to the list
     */
    @Override
    public void addReference(Field reference) {
        references.add(reference);
    }

    /**
     * Overwrites the methode from super class. Adds this instance of the class to the list.
     * 
     * @return the list of references
     * @throws IllegalAccessException never
     */
    @Override
    public LinkedList<Referenceable> getAsReferenceables() throws IllegalAccessException {
        LinkedList<Referenceable> list = new LinkedList<>();
        list.add(this);
        return list;
    }

    /**
     * Overwrites the methode from the interface and returns a list of all which are linked to the value stored in this class
     * 
     * @return the list of references
     */
    @Override
    public LinkedList<Field> getReferences() {
        return references;
    }
}
