package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.util.LinkedList;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;

/**
 * Interface implemented by classes which store a value from a bdt file and need a reference to the fields where this value is stored.
 * 
 * @param <E> the type of the value stored in the class
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public interface Referenceable<E> {
    /**
     * Returns a list of all fields which store the value.
     * 
     * @return the list of references
     */
    LinkedList<Field> getReferences();

    /**
     * Returns the value which was stored in the files.
     * 
     * @return the value
     */
    E getValue();

    /**
     * setter for the value which was stored in the files.
     * 
     * @param value the value
     */
    void setValue(E value);

    /**
     * adds a reference to the list of references where the value is stored.
     * 
     * @param reference the reference to a field
     */
    void addReference(Field reference);
}
