package de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord;

import java.lang.reflect.Field;
import java.util.LinkedList;

/**
 * Abstract class to implements a way to get all referenceable fields in the class
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public abstract class Referencer {
    /**
     * Returns all the referenceable fields in the class
     * 
     * @return a list of referenceable fields
     * @throws IllegalAccessException if #addFields throw the IllegalAccessException
     */
    public LinkedList<Referenceable> getAsReferenceables() throws IllegalAccessException {
        LinkedList<Referenceable> list = new LinkedList<Referenceable>();
        addFields(this.getClass().getDeclaredFields(), list, this);
        return list;
    }

    /**
     * Select all Fields with BDType from a given array of Fields and adds them to a given linked list
     * 
     * @param fields the array of all fields in the class
     * @param list the list to which the fields should be added
     * @param caller the reference from the calling class
     * @throws IllegalAccessException if a field is not accessible
     */
    public static void addFields(Field[] fields, LinkedList<Referenceable> list, Object caller) throws IllegalAccessException {
        for (Field field : fields) {
            field.setAccessible(true);
            if (field.getType() == BDTValue.class)
                list.add((Referenceable) field.get(caller));
            field.setAccessible(false);
        }
    }
}
