package de.uni_goettingen.med.mi.bdtparser.core.validator;

import static org.junit.Assert.fail;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.time.DateTimeException;
import java.util.HashSet;
import java.util.Set;

import org.junit.Before;
import org.junit.Test;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.BdtLine;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.Sentence;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.SentenceCharacteristic;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.sentence.SubSentence;

public class ValidatorCheckRuleTest {

    private static String METHOD_NAME = "checkRule";
    private Method methodCheckRule;
    private Object[] parameters;
    private Class[] parameterTypes;
    private Validator validator;

    @Before
    public void setUp() throws Exception {
        validator = new Validator();
        parameterTypes = new Class[2];
        parameterTypes[0] = int.class;
        parameterTypes[1] = de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field.class;
        methodCheckRule = validator.getClass().getDeclaredMethod(METHOD_NAME, parameterTypes);
        methodCheckRule.setAccessible(true);
        parameters = new Object[2];
    }

    @Test
    public final void testGetSubsentences() {
        for (SentenceCharacteristic sc : SentenceCharacteristic.values()) {
            try {
                for (SubSentence subsentence : sc.getSubSenteces()) {
                    subsentence.getFieldCharactersitic();
                }
            } catch (Exception e) {
                fail(e.getCause().toString());
            }

        }
    }

    /**
     * Test method for Validator#checkRule Checks if all rules from FieldCharacteristic are checked
     */
    @Test
    public final void testValidator() {
        Set<Integer> rules = getAllFieldCharacteristicRules();
        if (rules != null && !rules.isEmpty()) {
            for (Integer rule : rules) {
                Sentence sentence = new Sentence();
                Field field = new Field("00", "00", "00", sentence);
                rule = 323;
                parameters[0] = rule;
                parameters[1] = field;
                Boolean result = null;
                try {
                    result = (boolean) methodCheckRule.invoke(validator, parameters);
                } catch (IllegalAccessException iace) {
                    fail(iace.getCause().toString());
                } catch (InvocationTargetException ite) {
                    fail(ite.getCause().toString());
                } catch (IllegalArgumentException iare) {
                    fail(iare.getCause().toString());
                }
                // assertNotNull(result);
            }
        }
    }

    @Test
    public final void testCheckRule304() throws IllegalAccessException, IllegalArgumentException {
        BdtLine bdtLine = new BdtLine("017310300000000");
        Sentence sentence = new Sentence();
        Field field = new Field(bdtLine, sentence);
        int rule = 304;
        parameters[0] = rule;
        parameters[1] = field;
        Boolean result = null;
        try {
            result = (boolean) methodCheckRule.invoke(validator, parameters);
        } catch (InvocationTargetException ite) {
            if (ite.getCause() instanceof DateTimeException) {
                fail(ite.getCause().toString());
            }
        }

    }

    private Set<Integer> getAllFieldCharacteristicRules() {
        Set<Integer> result = new HashSet<>();
        for (FieldCharacteristic fieldCharacteristic : FieldCharacteristic.values()) {
            int[] rules = fieldCharacteristic.getRules();
            if (rules != null && rules.length > 0) {
                for (int i = 0; i < rules.length; i++) {
                    result.add(rules[i]);
                }
            }
        }
        return result;
    }

}
