package de.uni_goettingen.med.mi.bdtparser.core.validator;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;

import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;

import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Datatyp;
import de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field;

import static org.junit.Assert.*;

public class ValidatorFieldDatatypMatchTest {

    private static String METHOD_NAME = "fieldDatatypMatch";
    private Method methodFieldDatatypMatch;
    private Object[] parameters;
    private Class[] parameterTypes;
    private Validator validator;
    private ArrayList<TestSetEntry> lstTestDataSet;

    @Before
    public void setUp() throws Exception {
        validator = new Validator();
        parameterTypes = new Class[2];
        parameterTypes[0] = de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Field.class;
        parameterTypes[1] = de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.Datatyp.class;
        methodFieldDatatypMatch = validator.getClass().getDeclaredMethod(METHOD_NAME, parameterTypes);
        methodFieldDatatypMatch.setAccessible(true);
        parameters = new Object[2];
    }

    private void createTestDataSet() {
        lstTestDataSet = new ArrayList<TestSetEntry>();

        // Alphanumeric
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.ALNUM, ""));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.ALNUM, "xy008"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.ALNUM, "x0y0z"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.ALNUM, "0x0y8"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.ALNUM, "008xy"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.ALNUM, "00000"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.ALNUM, "10000"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.ALNUM, "abcde"));
        // Numeric
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.NUM, ""));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.NUM, "008"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.NUM, "8"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.NUM, "-4"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.NUM, "00000"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.NUM, "xy008"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.NUM, "x0y0z"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.NUM, "0x0y8"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.NUM, "008xy"));
        // Float
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.FLOAT, ""));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.FLOAT, "0.1"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.FLOAT, "10.5"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.FLOAT, "-0.5"));
//        lstTestDataSet.add(new TestSetEntry(false, Datatyp.FLOAT, "10,3"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.FLOAT, "xy008"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.FLOAT, "x0y0z"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.FLOAT, "0x0y8"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.FLOAT, "008xy"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.FLOAT, "00000"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.FLOAT, "10000"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.FLOAT, "abcde"));
        // Date
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.DATE, ""));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.DATE, "01012018"));
        lstTestDataSet.add(new TestSetEntry(true, Datatyp.DATE, "20180101"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.DATE, "000118"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.DATE, "01.01.2018"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.DATE, "01-01.2018"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.DATE, "10,3"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.DATE, "xy008"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.DATE, "x0y0z"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.DATE, "0x0y8"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.DATE, "008xy"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.DATE, "00000"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.DATE, "10000"));
        lstTestDataSet.add(new TestSetEntry(false, Datatyp.DATE, "abcde"));
    }

    /**
     * Test method for Validator#fieldDatatypMatch
     */
    @Test
    public final void testValidatorFieldDatatypMatch() {
        createTestDataSet();
        for (TestSetEntry testSetEntry : lstTestDataSet) {
            parameters[0] = testSetEntry.getField();
            parameters[1] = testSetEntry.getDatatyp();
            Boolean result = null;
            String logText = "Checked value: " + testSetEntry.getText() + " as " + testSetEntry.getDatatyp();
            try {
                result = (boolean) methodFieldDatatypMatch.invoke(validator, parameters);
            } catch (IllegalAccessException iace) {
                fail(logText + iace.getCause().toString());
            } catch (InvocationTargetException ite) {
                fail(logText + ite.getCause().toString());
            } catch (IllegalArgumentException iare) {
                fail(logText + iare.getCause().toString());
            }
            assertNotNull(result);
            assertEquals(logText, testSetEntry.getIsCorrect(), result);
        }
    }

}

class TestSetEntry {

    private Boolean isCorrect;
    private Datatyp datatype;
    private Field field;

    TestSetEntry(boolean isCorrect, Datatyp datatype, String fieldData) {
        this.isCorrect = isCorrect;
        this.datatype = datatype;
        this.field = new Field("000", "0000", fieldData, null);
    }

    public String getText() {
        return this.field.getData();
    }

    public Boolean getIsCorrect() {
        return this.isCorrect;
    }

    public Object getDatatyp() {
        return this.datatype;
    }

    public Object getField() {
        return this.field;
    }

}
