package de.uni_goettingen.med.mi.radar_software.model;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;

/**
 * Class to store all patient data needed for the pseudoymization process.
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class Patient {

    private StringProperty localSystemPatientId;
    private StringProperty firstName;
    private StringProperty lastName;
    private StringProperty birthDate;
    private StringProperty gender;
    private StringProperty city;
    private StringProperty zipCode;
    private StringProperty street;
    private StringProperty treatingPractice;
    private StringProperty pseudonym;
    private BooleanProperty consent;

    /**
     * Constructor that expects the id of the patient from the local system
     * @param localSystemPatientId the id
     */
    public Patient(String localSystemPatientId) {
        super();
        this.localSystemPatientId = new SimpleStringProperty(localSystemPatientId);
    }

    /**
     * Getter for the local ID
     * @return the id
     */
    public StringProperty localSystemPatientIdProperty() {
        return localSystemPatientId;
    }

    /**
     * Getter for the first name
     * @return the first name
     */
    public StringProperty firstNameProperty() {
        return firstName;
    }

    /**
     * Getter for the last name
     * @return the lasst name
     */
    public StringProperty lastNameProperty() {
        return lastName;
    }

    /**
     * Getter for the treating pratcie
     * @return the treating practice
     */
    public StringProperty treatingPracticeProperty() {
        return treatingPractice;
    }

    /**
     * Getter for the pseudonym
     * @return the the pseudonym
     */
    public StringProperty pseudonymProperty() {
        return pseudonym;
    }

    /**
     * Getter for the birth date
     * @return the birthdate
     */
    public StringProperty birthDateProperty() {
        return birthDate;
    }

    /**
     * Getter for the gender
     * @return the gender
     */
    public StringProperty genderProperty() {
        return gender;
    }

    /**
     * Getter for the city
     * @return the city
     */
    public StringProperty cityProperty() {
        return city;
    }

    /**
     * Getter for the zip code
     * @return the zip code
     */
    public StringProperty zipCodeProperty() {
        return zipCode;
    }

    /**
     * Getter for the street
     * @return the street
     */
    public StringProperty streetProperty() {
        return street;
    }

    /**
     * Getter for the consent
     * @return true if the consent is given, false otherwise
     */
    public BooleanProperty consentProperty() {
        return consent;
    }

    /**
     * Setter for the first name
     * @param firstName  the first name
     */
    public void setFirstName(String firstName) {
        this.firstName = new SimpleStringProperty(firstName);
    }

    /**
     * Setter for the last name
     * @param lastName  the last name
     */
    public void setLastName(String lastName) {
        this.lastName = new SimpleStringProperty(lastName);
    }

    /**
     * Setter for the treating practice
     * @param treatingPractice the practice
     */
    public void setTreatingPractice(String treatingPractice) {
        this.treatingPractice = new SimpleStringProperty(treatingPractice);
    }

    /**
     * setter for the pseudonym
     * @param pseudonym the pseudonym
     */
    public void setPseudonym(String pseudonym) {
        this.pseudonym = new SimpleStringProperty(pseudonym);
    }

    /**
     * Setter for the birth date
     * @param birthDate the birth date
     */
    public void setBirthDate(String birthDate) {
        this.birthDate = new SimpleStringProperty(birthDate);
    }

    /**
     * Setter for the gender
     * @param gender the gender
     */
    public void setGender(String gender) {
        this.gender = new SimpleStringProperty(gender);
    }

    /**
     * Setter for the city
     * @param city the city
     */
    public void setCity(String city) {
        this.city = new SimpleStringProperty(city);
    }

    /**
     * Setter for the zip code
     * @param zipCode the zip code
     */
    public void setZipCode(String zipCode) {
        this.zipCode = new SimpleStringProperty(zipCode);
    }

    /**
     * Setter for the street
     * @param street the street
     */
    public void setStreet(String street) {
        this.street = new SimpleStringProperty(street);
    }

    /**
     * Setter for the flag if the patient has given his consent
     * @param consent the flag
     */
    public void setConsent(Boolean consent) {
        this.consent = new SimpleBooleanProperty(consent);
    }

}