package de.uni_goettingen.med.mi.radar_software.ths.message;

import java.io.IOException;
import java.io.OutputStreamWriter;

import javax.net.ssl.HttpsURLConnection;

import org.json.simple.JSONObject;
import org.json.simple.parser.ParseException;

import de.uni_goettingen.med.mi.radar_software.ths.THSHelper;

/**
 * THS AbstractResponse
 * 
 * Code friendly adopted from DZHK colleagues (author Rottmann)
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class GetSessionRequest {
    String user_id;
    String user_name;
    String user_title = ""; // optional
    String user_firstname = ""; // optional
    String user_lastname = ""; // optional
    String user_role = ""; // optional

    public GetSessionRequest(String user_id, String user_name) {
        this.user_id = user_id;
        this.user_name = user_name;
    }

    @SuppressWarnings("unchecked")
    private JSONObject toJSON() {
        JSONObject fields = new JSONObject();
        fields.put("user_id", this.user_id);
        fields.put("user_name", this.user_name);
        fields.put("user_title", this.user_title);
        fields.put("user_firstname", this.user_firstname);
        fields.put("user_lastname", this.user_lastname);
        fields.put("user_role", this.user_role);

        JSONObject data = new JSONObject();
        data.put("fields", fields);

        JSONObject json = new JSONObject();
        json.put("data", data);

        return json;
    }

    public GetSessionResponse request(THSHelper helper) throws IOException, ParseException {
        HttpsURLConnection con = helper.getConnection();
        OutputStreamWriter post = new OutputStreamWriter(con.getOutputStream(),"UTF-8");

        this.toJSON().writeJSONString(post);
        post.flush();
        post.close();
        
        return (new GetSessionResponse(con));
    }

    @Override
    public String toString() {
        return this.toJSON().toJSONString();
    }

}