package de.uni_goettingen.med.mi.radar_software.ths.message;

import java.io.IOException;
import java.io.OutputStreamWriter;

import javax.net.ssl.HttpsURLConnection;

import org.json.simple.JSONObject;
import org.json.simple.parser.ParseException;

import de.uni_goettingen.med.mi.radar_software.ths.THSHelper;

/**
 * THS GetTokenRequest
 * 
 * Code friendly adopted from DZHK colleagues (author Rottmann)
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class GetTokenRequest {

    public enum TokenType {
        requestPSN, addPatient, updateIDAT, addConsentByPatient
    }

    public enum TokenMethod {
        getOrCreate, addPatientWithConsentAIS
    }

    public enum TokenEvent {
        requestPSNTemp, requestPSNBsnrTemp, addPatientAIS, addPatientWithConsentAIS, updateIDATForTabletPatient, requestAisPsnByPatId, addConsentAIS
    }

    public enum TokenTargetType {
        temp, bsnr_temp
    }

    public enum TokenTargetIdType {
        ais
    }

    public enum TokenReason {
        generateTempFromAisForDb, generateTempFromBsnrForDb, generateAisPsnByPatId
    }

    TokenType type;
    TokenMethod method;
    TokenEvent event;
    TokenTargetType targetType;
    TokenTargetIdType targetIdType;
    TokenReason reason;

    String sessionId;
    String uri;

    String study_id;
    String study_name;

    String location_id;
    String location_name;

    public GetTokenResponse request(THSHelper helper) throws IOException, ParseException {
        HttpsURLConnection con = helper.getConnection(this.uri);
        OutputStreamWriter post = new OutputStreamWriter(con.getOutputStream(),"UTF-8");

        this.toJSON().writeJSONString(post);
        post.flush();
        post.close();

        return (new GetTokenResponse(con));
    }

    public GetTokenRequest(GetSessionResponse sessionResponse) {
        this.sessionId = sessionResponse.getSessionId();
        this.uri = sessionResponse.getUri() + "/tokens";
    }

    @SuppressWarnings("unchecked")
    private JSONObject toJSON() {
        JSONObject fields = new JSONObject();
        fields.put("study_id", this.study_id);
        fields.put("study_name", this.study_name);
        if (this.location_id != null)
            fields.put("location_id", this.location_id);
        if (this.location_name != null)
            fields.put("location_name", this.location_name);
        fields.put("event", this.event.toString());
        if (this.targetIdType != null)
            fields.put("targetIdType", this.targetIdType.toString());
        if (this.targetType != null)
            fields.put("targetType", this.targetType.toString());
        if (this.reason != null)
            fields.put("reason", this.reason.toString());

        JSONObject data = new JSONObject();
        data.put("fields", fields);

        JSONObject json = new JSONObject();
        json.put("type", this.type.toString());
        if (this.method != null)
            json.put("method", this.method.toString());
        json.put("data", data);

        return json;
    }

    public String getSessionId() {
        return this.sessionId;
    }

    public String getUri() {
        return this.uri;
    }

    public void setType(TokenType type) {
        this.type = type;
    }

    public void setMethod(TokenMethod method) {
        this.method = method;
    }

    public void setEvent(TokenEvent event) {
        this.event = event;
    }

    public void setTargetType(TokenTargetType targettype) {
        this.targetType = targettype;
    }

    public void setTargetIdType(TokenTargetIdType targetidtype) {
        this.targetIdType = targetidtype;
    }

    public void setStudyId(String studyid) {
        this.study_id = studyid;
    }

    public void setStudyName(String studyname) {
        this.study_name = studyname;
    }

    public void setReason(TokenReason reason) {
        this.reason = reason;
    }

    public String getLocation_id() {
        return location_id;
    }

    public void setLocation_id(String location_id) {
        this.location_id = location_id;
    }

    public String getLocation_name() {
        return location_name;
    }

    public void setLocation_name(String location_name) {
        this.location_name = location_name;
    }

    @Override
    public String toString() {
        return this.toJSON().toJSONString();
    }

}