package de.uni_goettingen.med.mi.radar_software.ths.message;

import java.io.IOException;
import java.util.HashMap;

import javax.net.ssl.HttpsURLConnection;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.ParseException;

/**
 * THS RequestAddConsentResponse
 * 
 * Code friendly adopted from DZHK colleagues (author Rottmann)
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class RequestAddConsentResponse extends AbstractResponse {

	private static final Logger logger = LogManager.getLogger(RequestAddConsentResponse.class);

	RequestAddConsentResponse(HttpsURLConnection request) throws IOException, ParseException {
		super(request);
	}

	public boolean containsErrors() {
		return this.response.toString().contains("errors");
	}

	public HashMap<String, String> getErrorCodes() {
		HashMap<String, String> errorCodes = new HashMap<>();
		logger.debug(this.response.toJSONString());
		if (this.containsErrors()) {
			JSONArray errorList = (JSONArray) this.response.get("errors");
			for (int i = 0; i < errorList.size(); i++) {
				JSONObject jsonError = (JSONObject) errorList.get(i);
				@SuppressWarnings("unused")
				String code = (String) jsonError.get("code");
				String id = (String) jsonError.get("id");
				String message = (String) jsonError.get("message");
				@SuppressWarnings("unused")
				String referencedItem = (String) jsonError.get("referencedItem");

				errorCodes.put(id, message);
			}
		}
		return errorCodes;
	}

}