package de.uni_goettingen.med.mi.radar_software.ths.message;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import javax.net.ssl.HttpsURLConnection;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.ParseException;

import de.uni_goettingen.med.mi.radar_software.ths.THSHelper;

/**
 * THS RequestAddPatient
 * 
 * Code friendly adopted from DZHK colleagues (author Rottmann)
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class RequestAddPatient {
	
    public enum RequestProcessDomain {
        bsnr_orig, ais, RADAR
    }

	RequestProcessDomain processDomain;
	private String psnName;
	private String bsnr;

	String tokenId;
	String url;
	List<THSPatient> thsPatients;
	private HashMap<String, String> hmAisPsn;

	public RequestAddPatient(GetTokenResponse tokenResponse) {
		this.tokenId = tokenResponse.getTokenId();
		this.url = tokenResponse.getUrl();
	}

	public String getTokenId() {
		return this.tokenId;
	}

	public String getUrl() {
		return this.url;
	}

	public void setPsnDomain(RequestProcessDomain processDomain) {
		this.processDomain = processDomain;
	}

	public void setPsnName(String psnName) {
		this.psnName = psnName;
	}

	public void setPatients(List<THSPatient> thsPatients) {
		this.thsPatients = thsPatients;
	}

	public void setAisPsn(HashMap<String, String> hmAisPsn) {
		this.hmAisPsn = hmAisPsn;
	}

	public void setBsnr(String bsnr) {
		this.bsnr = bsnr;
	}

	/**
	 *
	 * @return JSONObject following the structure defines in THS interface specification
	 */
	@SuppressWarnings("unchecked")
	private JSONObject toJSON() {
		JSONObject json = new JSONObject();
		JSONArray patients = new JSONArray();

		switch (processDomain) {
		case RADAR:
			if (thsPatients != null) {
				for (THSPatient thsPatient : thsPatients) {

					JSONObject pat = new JSONObject();

					// patient
					pat.put("index", Integer.toString(thsPatient.getIndex()));
					JSONObject patient = new JSONObject();
					JSONArray identifiers = new JSONArray();
					JSONObject identifier = new JSONObject();
					identifiers.add(identifier);
					identifier.put("domain", this.processDomain.toString());
					identifier.put("name", this.psnName);
					identifier.put("id", thsPatient.getIdentifier_id());
					patient.put("identifier", identifiers);
					patient.put("firstName", thsPatient.getFirstName());
					patient.put("lastName", thsPatient.getLastName());
					patient.put("birthdate", thsPatient.getBirthdate());
					patient.put("gender", thsPatient.getGender());
					patient.put("epix_value1", thsPatient.getTreatingPractice());
					pat.put("patient", patient);

					// contacts
					JSONArray contacts = new JSONArray();
					JSONObject contact = new JSONObject();
					contacts.add(contact);
					contact.put("city", thsPatient.getCity());
					contact.put("street", thsPatient.getStreet());
					contact.put("zipCode", thsPatient.getZipCode());
					pat.put("contacts", contacts);

					patients.add(pat);

				}
			}
            break;
        case bsnr_orig:
			if (bsnr != null) {
				JSONObject pat = new JSONObject();
				// patient
				pat.put("index", "1");
				JSONObject patientIdentifier = new JSONObject();
				patientIdentifier.put("domain", this.processDomain.toString());
				patientIdentifier.put("name", "PSN");
				patientIdentifier.put("id", this.bsnr);
				patientIdentifier.put("type", "patientPSN");
				pat.put("patientIdentifier", patientIdentifier);
				patients.add(pat);
			}
			break;
		case ais:
			if (hmAisPsn != null) {
				for (Entry<String, String> entry : hmAisPsn.entrySet()) {
					String key = entry.getKey();
					String value = entry.getValue();

					JSONObject pat = new JSONObject();
					// patient
					pat.put("index", key);
					JSONObject patientIdentifier = new JSONObject();
					patientIdentifier.put("domain", this.processDomain.toString());
					patientIdentifier.put("name", "PSN");
					patientIdentifier.put("id", value);
					patientIdentifier.put("type", "patientPSN");
					pat.put("patientIdentifier", patientIdentifier);
					patients.add(pat);
				}
			}
			break;
		}
		json.put("patients", patients);
		return json;
	}

	public RequestAddPatientResponse request(THSHelper helper) throws IOException, ParseException {
		HttpsURLConnection con = helper.getConnection(this.url);
		OutputStreamWriter post = new OutputStreamWriter(con.getOutputStream(), "UTF-8");

		this.toJSON().writeJSONString(post);
		post.flush();
		post.close();

		return (new RequestAddPatientResponse(con));
	}

	@Override
	public String toString() {
		return this.toJSON().toJSONString();
	}

}
