package de.uni_goettingen.med.mi.radar_software.ths.message;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import javax.net.ssl.HttpsURLConnection;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.ParseException;

import de.uni_goettingen.med.mi.radar_software.ths.THSHelper;

/**
 * THS RequestPSNRequest
 * 
 * Code friendly adopted from DZHK colleagues (author Rottmann)
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class RequestPSNRequest {
	
    public enum RequestProcessDomain {
        bsnr_orig, RADAR, ais
    }

	RequestProcessDomain processDomain;
	private String psnName;
	private String bsnr;

	String tokenId;
	String url;
	List<THSPatient> thsPatients;
	private HashMap<String, String> hmAisPsn;

	public RequestPSNRequest(GetTokenResponse tokenResponse) {
		this.tokenId = tokenResponse.getTokenId();
		this.url = tokenResponse.getUrl();
	}

	public String getTokenId() {
		return this.tokenId;
	}

	public String getUrl() {
		return this.url;
	}

	public void setPsnDomain(RequestProcessDomain processDomain) {
		this.processDomain = processDomain;
	}

	public void setPsnName(String psnName) {
		this.psnName = psnName;
	}

	public void setPatients(List<THSPatient> thsPatients) {
		this.thsPatients = thsPatients;
	}

	public void setAisPsn(HashMap<String, String> hmAisPsn) {
		this.hmAisPsn = hmAisPsn;
	}

	public void setBsnr(String bsnr) {
		this.bsnr = bsnr;
	}

	/**
	 *
	 * @return JSONObject following the structure defines in THS interface specification
	 */
	@SuppressWarnings("unchecked")
	private JSONObject toJSON() {
		JSONObject json = new JSONObject();
		JSONArray patients = new JSONArray();

        switch (processDomain) {
        case bsnr_orig:
            if (bsnr != null) {
                JSONObject pat = new JSONObject();
                // patient
                pat.put("index", "1");
                JSONObject patientIdentifier = new JSONObject();
                patientIdentifier.put("domain", this.processDomain.toString());
                patientIdentifier.put("name", this.psnName);
                patientIdentifier.put("id", this.bsnr);
                patientIdentifier.put("type", "patientPSN");
                pat.put("patientIdentifier", patientIdentifier);
                patients.add(pat);
            }
            break;
        case RADAR:
            if (thsPatients != null) {
                for (THSPatient thsPatient : thsPatients) {
                    JSONObject pat = new JSONObject();
                    // patient
                    pat.put("index", Integer.toString(thsPatient.getIndex()));
                    JSONObject patientIdentifier = new JSONObject();
                    patientIdentifier.put("domain", this.processDomain.toString());
                    patientIdentifier.put("name", this.psnName);
                    patientIdentifier.put("id", thsPatient.getIdentifier_id());
                    patientIdentifier.put("type", "localIdentifier");
                    pat.put("patientIdentifier", patientIdentifier);
                    patients.add(pat);
                }
            }
            break;
        case ais:
            if (hmAisPsn != null) {
                for (Entry<String, String> entry : hmAisPsn.entrySet()) {
                    String key = entry.getKey();
                    String value = entry.getValue();

                    JSONObject pat = new JSONObject();
                    // patient
                    pat.put("index", key);
                    JSONObject patientIdentifier = new JSONObject();
                    patientIdentifier.put("domain", this.processDomain.toString());
                    patientIdentifier.put("name", this.psnName);
                    patientIdentifier.put("id", value);
                    patientIdentifier.put("type", "patientPSN");
                    pat.put("patientIdentifier", patientIdentifier);
                    patients.add(pat);
                }
            }
            break;
        }
		json.put("patients", patients);
		return json;
	}

	public RequestPSNResponse request(THSHelper helper) throws IOException, ParseException {
		HttpsURLConnection con = helper.getConnection(this.url);
		OutputStreamWriter post = new OutputStreamWriter(con.getOutputStream(), "UTF-8");

		this.toJSON().writeJSONString(post);
		post.flush();
		post.close();

		return (new RequestPSNResponse(con));
	}

	@Override
	public String toString() {
		return this.toJSON().toJSONString();
	}

}
