package de.uni_goettingen.med.mi.radar_software.ths.message;

import java.io.IOException;
import java.util.HashMap;

import javax.net.ssl.HttpsURLConnection;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.ParseException;

/**
 * THS RequestPSNResponse
 * 
 * Code friendly adopted from DZHK colleagues (author Rottmann)
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class RequestPSNResponse extends AbstractResponse {

	private static final Logger logger = LogManager.getLogger(RequestPSNResponse.class);

	RequestPSNResponse(HttpsURLConnection request) throws IOException, ParseException {
		super(request);
	}

	public boolean containsErrors() {
		return this.response.toString().contains("errors");
	}

	public HashMap<String, String> getErrorCodes() {
		HashMap<String, String> errorCodes = new HashMap<>();
		logger.debug(this.response.toJSONString());
		if (this.containsErrors()) {
			JSONArray errorList = (JSONArray) this.response.get("errors");
			for (int i = 0; i < errorList.size(); i++) {
				JSONObject jsonError = (JSONObject) errorList.get(i);
				@SuppressWarnings("unused")
				String code = (String) jsonError.get("code");
				String id = (String) jsonError.get("id");
				String message = (String) jsonError.get("message");
				@SuppressWarnings("unused")
				String referencedItem = (String) jsonError.get("referencedItem");

				errorCodes.put(id, message);
			}
		}
		return errorCodes;
	}

	public HashMap<String, String> getPsnList() {
		HashMap<String, String> patientIdentifiers = new HashMap<>();
		JSONArray psnList = (JSONArray) this.response.get("psnList");
		if (psnList != null) {
			for (int i = 0; i < psnList.size(); i++) {
				JSONObject jsonPatient = (JSONObject) psnList.get(i);
				String index = (String) jsonPatient.get("index");
				String targetId = (String) jsonPatient.get("targetId");
				Boolean tentative = (Boolean) jsonPatient.get("tentative");
				String patientStatus = (String) jsonPatient.get("patientStatus");
				patientIdentifiers.put(index, targetId);
			}
		}
		return patientIdentifiers;
	}

	public HashMap<String, String> getPatientIdentifiers() {
		HashMap<String, String> patientIdentifiers = new HashMap<>();
		JSONArray psnList = (JSONArray) this.response.get("patients");
		if (psnList != null) {
			for (int i = 0; i < psnList.size(); i++) {
				JSONObject jsonPatient = (JSONObject) psnList.get(i);
				String index = (String) jsonPatient.get("index");
				String targetId = (String) jsonPatient.get("targetId");
				patientIdentifiers.put(index, targetId);
			}
		}
		return patientIdentifiers;
	}

	public String getBsnrPsn() {
		String result = "";
		JSONArray psnList = (JSONArray) this.response.get("patients");
		if (psnList != null) {
			for (int i = 0; i < psnList.size(); i++) {
				JSONObject jsonPatient = (JSONObject) psnList.get(i);
				String targetId = (String) jsonPatient.get("targetId");
				result = targetId;
			}
		}
		return result;
	}

}