package de.uni_goettingen.med.mi.radar_software.ths.message;

import java.io.IOException;
import java.util.HashMap;

import javax.net.ssl.HttpsURLConnection;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.ParseException;

/**
 * THS RequestUpdateIDATResponse
 * 
 * Code friendly adopted from DZHK colleagues (author Rottmann)
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class RequestUpdateIDATResponse extends AbstractResponse {

	private static final Logger logger = LogManager.getLogger(RequestUpdateIDATResponse.class);

	RequestUpdateIDATResponse(HttpsURLConnection request) throws IOException, ParseException {
		super(request);
	}

	public boolean containsErrors() {
		return this.response.toString().contains("errorCode");
	}

	public HashMap<String, String> getErrorCodes() {
		HashMap<String, String> errorCodes = new HashMap<>();
		logger.debug(this.response.toJSONString());
		if (this.containsErrors()) {
			JSONArray errorList = (JSONArray) this.response.get("patients");
			for (int i = 0; i < errorList.size(); i++) {
				JSONObject jsonPatientError = (JSONObject) errorList.get(i);
				String index = (String) jsonPatientError.get("index");
				String errorCode = (String) jsonPatientError.get("errorCode");
				errorCodes.put(index, errorCode);
			}
		}
		return errorCodes;
	}

}