package de.uni_goettingen.med.mi.radar_software.ths.message;

import java.util.HashMap;

import de.uni_goettingen.med.mi.radar_software.model.InformedConsent.InformedConsentSection;

/**
 * Class used to store all the data of the patient for to transfer it to the trust authority for pseudonymisation
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class THSPatient {

    // patient
    private int index;
    private String identifier_id;
    private String firstName;
    private String lastName;
    private String birthdate;
    private String gender;
    // contacts
    private String city;
    private String zipCode;
    private String street;
    // consents
    private HashMap<InformedConsentSection, Boolean> hmConsentModules;
    // treating practice
    private String treatingPractice;

    /**
     * Constructor
     * @param identifier_id the id of the patient
     */
    public THSPatient(String identifier_id) {
        super();
        this.identifier_id = identifier_id;
        this.hmConsentModules = new HashMap<InformedConsentSection, Boolean>();
    }

    public int getIndex() {
        return index;
    }

    public void setIndex(int index) {
        this.index = index;
    }

    public String getIdentifier_id() {
        return identifier_id;
    }

    public void setIdentifier_id(String identifier_id) {
        this.identifier_id = identifier_id;
    }

    public String getFirstName() {
        return firstName;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public String getBirthdate() {
        return birthdate;
    }

    // THS Format YYYY-MM-DD (e.g. 1990-01-01)
    public void setBirthdate(String birthdate) {
        this.birthdate = birthdate;
    }

    public String getGender() {
        return gender;
    }

    public void setGender(String gender) {
        this.gender = gender;
    }

    public String getCity() {
        return city;
    }

    public void setCity(String city) {
        this.city = city;
    }

    public String getZipCode() {
        return zipCode;
    }

    public void setZipCode(String zipCode) {
        this.zipCode = zipCode;
    }

    public String getStreet() {
        return street;
    }

    public void setStreet(String street) {
        this.street = street;
    }

    public String getTreatingPractice() {
        return treatingPractice;
    }

    public void setTreatingPractice(String treatingPractice) {
        this.treatingPractice = treatingPractice;
    }

    public void addConsentModule(InformedConsentSection icSection, Boolean consentModuleStatus) {
        hmConsentModules.put(icSection, consentModuleStatus);
    }

    public HashMap<InformedConsentSection, Boolean> getConsentModule() {
        return hmConsentModules;
    }

}