package de.uni_goettingen.med.mi.radar_software.util;

import java.util.HashMap;

import de.uni_goettingen.med.mi.radar_software.util.LogAnalyser.ErrorType;

/**
 * Class used while analying the generated logs. It is a extend hash map that also stores the typ of errors that are stored in the map
 * and a short description where thees errors occured.
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class ErrorMap<K, V> extends HashMap {
    public final String Description;
    private ErrorType errorType;

    /**
     * Constructor that just required the description
     * @param description the description of the error types
     */
    public ErrorMap(String description) {
        super();
        Description = description;
    }

    /**
     * Constructor that used the description and the error type
     * @param description the description of the error types
     * @param errorType the error type
     */
    public ErrorMap(String description, ErrorType errorType) {
        this(description);
        setErrorType(errorType);
    }

    /**
     * Getter for the error type
     * @return the error type
     */
    public ErrorType getErrorType() {
        return errorType;
    }

    /**
     * Setter for the error type
     * @param errorType the error type
     */
    public void setErrorType(ErrorType errorType) {
        this.errorType = errorType;
    }
}
