package de.uni_goettingen.med.mi.radar_software.view;

import de.uni_goettingen.med.mi.radar_software.model.InformedConsent;
import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.RadioButton;
import javafx.scene.control.TextField;
import javafx.scene.control.ToggleGroup;
import javafx.stage.Stage;

/**
 * The controller for the informed consent edit view.
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class InformedConsentEditDialogController {

    @FXML
    private TextField patientIDField;
    @FXML
    private TextField firstNameField;
    @FXML
    private TextField lastNameField;
    @FXML
    private RadioButton consentSection1Yesrbtn;
    @FXML
    private RadioButton consentSection1Norbtn;
    @FXML
    private RadioButton consentSection2Yesrbtn;
    @FXML
    private RadioButton consentSection2Norbtn;
    @FXML
    private RadioButton consentSection3Yesrbtn;
    @FXML
    private RadioButton consentSection3Norbtn;
    @FXML
    private RadioButton consentSection4Yesrbtn;
    @FXML
    private RadioButton consentSection4Norbtn;

    private Stage dialogStage;
    private InformedConsent informedConsent;
    private boolean okClicked = false;

    /**
     * Method to iniatalze  the dialog view for the consent dialog.
     */
    @FXML
    private void initialize() {
        ToggleGroup toggleGroupConsentSection1 = new ToggleGroup();
        consentSection1Yesrbtn.setToggleGroup(toggleGroupConsentSection1);
        consentSection1Norbtn.setToggleGroup(toggleGroupConsentSection1);

        ToggleGroup toggleGroupConsentSection2 = new ToggleGroup();
        consentSection2Yesrbtn.setToggleGroup(toggleGroupConsentSection2);
        consentSection2Norbtn.setToggleGroup(toggleGroupConsentSection2);

        ToggleGroup toggleGroupConsentSection3 = new ToggleGroup();
        consentSection3Yesrbtn.setToggleGroup(toggleGroupConsentSection3);
        consentSection3Norbtn.setToggleGroup(toggleGroupConsentSection3);

        ToggleGroup toggleGroupConsentSection4 = new ToggleGroup();
        consentSection4Yesrbtn.setToggleGroup(toggleGroupConsentSection4);
        consentSection4Norbtn.setToggleGroup(toggleGroupConsentSection4);

    }

    public void setDialogStage(Stage dialogStage) {
        this.dialogStage = dialogStage;
    }

    /**
     * set the consent accordingly to the dialog result.
     * @param informedConsent
     */
    public void setInformedConsent(InformedConsent informedConsent) {
        this.informedConsent = informedConsent;
        patientIDField.setText(informedConsent.getLocalSystemPatientId());
        firstNameField.setText(informedConsent.getFirstLetterFirstName());
        lastNameField.setText(informedConsent.getFirstLetterLastName());
        if (informedConsent.isConsentErhebungVerarbeitungNutzung()) {
            consentSection1Yesrbtn.setSelected(true);
        } else {
            consentSection1Norbtn.setSelected(true);
        }
        if (informedConsent.isConsentErneuteKontaktaufnahme()) {
            consentSection2Yesrbtn.setSelected(true);
        } else {
            consentSection2Norbtn.setSelected(true);
        }
        if (informedConsent.isConsentGeltungsdauer()) {
            consentSection3Yesrbtn.setSelected(true);
        } else {
            consentSection3Norbtn.setSelected(true);
        }
        if (informedConsent.isConsentWiderrufsrecht()) {
            consentSection4Yesrbtn.setSelected(true);
        } else {
            consentSection4Norbtn.setSelected(true);
        }
    }

    public boolean isOkClicked() {
        return okClicked;
    }

    /**
     * Method called when the ok button was clicked.
     */
    @FXML
    private void handleOk() {
        if (isInputValid()) {
            informedConsent.addInformedConsent(patientIDField.getText(), firstNameField.getText(), lastNameField.getText());

            if (consentSection1Yesrbtn.isSelected()) {
                informedConsent.setConsentErhebungVerarbeitungNutzung(true);
            } else {
                informedConsent.setConsentErhebungVerarbeitungNutzung(false);
            }
            if (consentSection2Yesrbtn.isSelected()) {
                informedConsent.setConsentErneuteKontaktaufnahme(true);
            } else {
                informedConsent.setConsentErneuteKontaktaufnahme(false);
            }
            if (consentSection3Yesrbtn.isSelected()) {
                informedConsent.setConsentGeltungsdauer(true);
            } else {
                informedConsent.setConsentGeltungsdauer(false);
            }
            if (consentSection4Yesrbtn.isSelected()) {
                informedConsent.setConsentWiderrufsrecht(true);
            } else {
                informedConsent.setConsentWiderrufsrecht(false);
            }
            okClicked = true;
            dialogStage.close();
        }
    }

    @FXML
    private void handleCancel() {
        dialogStage.close();
    }

    /**
     * Valide the input from the user dialog.
     * @return
     */
    private boolean isInputValid() {
        String errorMessage = "";

        if (patientIDField.getText() == null || patientIDField.getText().length() == 0) {
            errorMessage += "Keine gültige Patienten-ID!\n";
        }
        if (firstNameField.getText() == null || firstNameField.getText().length() == 0) {
            errorMessage += "Kein gültiger Vorname!\n";
        }
        if (lastNameField.getText() == null || lastNameField.getText().length() == 0) {
            errorMessage += "Kein gültiger Nachname!\n";
        }
        if (!consentSection1Yesrbtn.isSelected() && !consentSection1Norbtn.isSelected()) {
            errorMessage += "Keine Auswahl zur Teilnahme in Abschnitt 1 getroffen!\n";
        }
        if (!consentSection2Yesrbtn.isSelected() && !consentSection2Norbtn.isSelected()) {
            errorMessage += "Keine Auswahl zur Teilnahme in Abschnitt 2 getroffen!\n";
        }
        if (!consentSection3Yesrbtn.isSelected() && !consentSection3Norbtn.isSelected()) {
            errorMessage += "Keine Auswahl zur Teilnahme in Abschnitt 3 getroffen!\n";
        }
        if (!consentSection4Yesrbtn.isSelected() && !consentSection4Norbtn.isSelected()) {
            errorMessage += "Keine Auswahl zur Teilnahme in Abschnitt 4 getroffen!\n";
        }
        if (errorMessage.length() == 0) {
            return true;
        } else {
            Alert alert = new Alert(AlertType.ERROR);
            alert.initOwner(dialogStage);
            alert.setTitle("Unvollständige Angaben");
            alert.setHeaderText("Bitte Felder kontrollieren");
            alert.setContentText(errorMessage);
            alert.showAndWait();
            return false;
        }
    }

}