package de.uni_goettingen.med.mi.radar_software.view;

import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.AUWegen;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.AbrechnungsartSchein;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Abrechnungsdiagnose;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Aerztemuster;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.AktuelleDiagnose;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.AnzahlAerzte;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.ArztgruppeVerbal;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Arztname;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Arztnummer;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.AusserhalbRezeptVerordnetesMedikament;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Befundart;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Dauerdiagnosen;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.DauerdiagnosenAbDatum;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Dauermedikamente;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.DauermedikamenteAbDatum;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Einheit;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Ergebnis_Wert;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.GNR_GNR_Ident;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Gebuehrenordnung;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.GeburtsdatumDesPatienten;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Geschaeftsstelle;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.GeschlechtDesPatienten;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.ICD_Schluessel;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.KrankenhauseinweisungWegen;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.MedikamentVerordnetAufRezept;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.PLZOrt;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Patientennummer_Patientenkennung;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.PhysikalischeTherapie;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Praxistyp;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.QuartalDerAbrechnung;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Test_Ident;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Testbezeichnung;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.Therapie;
import static de.uni_goettingen.med.mi.bdtparser.core.bdt_structur.field.FieldCharacteristic.UeberweisungInhalt;


import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Map.Entry;

import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.*;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_goettingen.med.mi.radar_software.MainApp;
import de.uni_goettingen.med.mi.bdtparser.core.virtualPatientRecord.BDTCase.CaseTypes;
import de.uni_goettingen.med.mi.radar_software.model.InformedConsent;
import javafx.application.HostServices;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.TreeItem;
import javafx.scene.control.TreeView;

/**
 * The controller for the summary view.
 * 
 * @author Department of Medical Informatics of the University Medical Center Göttingen (Hügel/Pung)
 */
public class SummaryController implements WizardController {

    private static final Logger log = LogManager.getLogger(SummaryController.class);
    private boolean viewerMode = false;
    private MainApp mainApp;
    private BDTArztpraxis bdtData;

    @FXML
    private TreeView<String> radarDataTreeView;

    @FXML
    private void initialize() {
        log.debug("initialized");
    }

    @Override
    public void setMainApp(MainApp mainApp) {
        this.mainApp = mainApp;
        log.debug("got " + this.mainApp);
        if (this.mainApp != null && !viewerMode) {
            this.bdtData = mainApp.getBdtData();
            filterForConsentPatients();
            makeBirthYear();
            drawSummaryTreeView();
        }
    }

    public void triggerDrawTree(MainApp mainApp){
        this.mainApp = mainApp;
        this.bdtData = mainApp.getBdtData();
        makeBirthYear();
        drawSummaryTreeView();
    }

    @Override
    public void setHostServices(HostServices hostServices) {
    }
    
    void activateViewerMode() {
        this.viewerMode = true;
    }

    /**
     * remove patients from the patient map that haven give their consent
     */
    private void filterForConsentPatients() {
        BDTArztpraxis bdtData = mainApp.getBdtData();
        if (bdtData != null) {
            HashMap<Long, BDTPatient> patientHashMap = bdtData.getPatients();
            for (Iterator<Entry<Long, BDTPatient>> it = patientHashMap.entrySet().iterator(); it.hasNext();) {
                Map.Entry<Long, BDTPatient> entry = it.next();
                BDTPatient bdtPatient = entry.getValue();
                String patNr = "";
                Long patientNummerKey = bdtPatient.getPatientNummerKey();
                if (patientNummerKey != null) {
                    patNr = bdtPatient.getPatientNummerKey().toString();
                }
                String firstName = "";
                BDTValue<String> firstNameValue = bdtPatient.getVorName();
                if (firstNameValue != null) {
                    firstName = firstNameValue.getValue();
                }
                String lastName = "";
                BDTValue<String> lastNameValue = bdtPatient.getNachName();
                if (lastNameValue != null) {
                    lastName = lastNameValue.getValue();
                }
                if (!checkForConsent(patNr, firstName, lastName)) {
                    it.remove();
                }
            }
        }
    }

    private void makeBirthYear() {
        BDTArztpraxis bdtData = mainApp.getBdtData();
        if (bdtData != null) {
            HashMap<Long, BDTPatient> patientHashMap = bdtData.getPatients();
            for (BDTPatient bdtPatient : patientHashMap.values()) {
                BDTValue<String> geburtsDatum = bdtPatient.getGeburtsDatum();
                if (geburtsDatum != null) {
                    bdtPatient.getGeburtsDatum().setValue(Integer.toString(bdtPatient.getBirthYear()));
                }
            }
        }
    }

    private Boolean checkForConsent(String patNr, String firstName, String lastName) {
        ObservableList<InformedConsent> lstInformedConsent = mainApp.getInformedConsentData();
        if (!patNr.isEmpty() && !firstName.isEmpty() && !lastName.isEmpty()) {
            for (InformedConsent ic : lstInformedConsent) {
                String localSystemPatientId = ic.getLocalSystemPatientId();
                String firstLetterFirstName = ic.getFirstLetterFirstName();
                String firstLetterLastName = ic.getFirstLetterLastName();
                if (localSystemPatientId.equals(patNr) && firstLetterFirstName.equals(firstName.substring(0, 1)) && firstLetterLastName.equals(lastName.substring(0, 1))) {
                    if (ic.isConsentErhebungVerarbeitungNutzung()) {
                        return Boolean.TRUE;
                    }
                }
            }
        }
        return Boolean.FALSE;
    }

    private void drawSummaryTreeView() {
        TreeItem<String> rootItem = new TreeItem<>("BDT-Daten");
        rootItem.setExpanded(true);
        if (bdtData != null) {
            TreeItem<String> item0010 = new TreeItem<>("Praxisdaten");
            rootItem.getChildren().add(item0010);

            Integer arztNummer = 0;
            if (bdtData.getArztNummer() != null)
                arztNummer = bdtData.getArztNummer().getValue();
            TreeItem<String> item0201 = new TreeItem<>(Arztnummer.getNumber() + " Arztnummer = " + arztNummer);
            item0010.getChildren().add(item0201);

            Integer praxistyp = 0;
            if (bdtData.getTyp() != null)
                praxistyp = bdtData.getTyp().getValue();
            TreeItem<String> item0202 = new TreeItem<>(Praxistyp.getNumber() + " Praxistyp = " + praxistyp);
            item0010.getChildren().add(item0202);

            LinkedList<BDTArzt> lstBdtAerzte = bdtData.getAerzte();
            if (lstBdtAerzte != null) {
                for (BDTArzt bdtArzt : lstBdtAerzte) {
                    if (bdtArzt != null) {
                        BDTValue<String> arztname = bdtArzt.getName();
                        if (arztname != null) {
                            String arzt = arztname.getValue();
                            arzt = ""; // not part of the mdat-list
                            TreeItem<String> item0203 = new TreeItem<>(Arztname.getNumber() + " Arztname " + arzt);
                            item0010.getChildren().add(item0203);
                            BDTValue<String> arztgruppe = bdtArzt.getArztgruppe();
                            if (arztgruppe != null) {
                                String arztgruppeVerbal = "";
                                arztgruppeVerbal = arztgruppe.getValue();
                                TreeItem<String> item0204 = new TreeItem<>(ArztgruppeVerbal.getNumber() + " Arztgruppe verbal = " + arztgruppeVerbal);
                                item0203.getChildren().add(item0204);
                            }
                        }
                    }
                }
            }

            String plzOrt = "";
            if (bdtData.getOrtPLZ() != null)
                plzOrt = bdtData.getOrtPLZ().getValue();
            TreeItem<String> item0206 = new TreeItem<>(PLZOrt.getNumber() + " PLZ Ort = " + plzOrt);
            item0010.getChildren().add(item0206);

            Integer anzahlAerzte = 0;
            if (bdtData.getAnzahlAerzte() != null)
                anzahlAerzte = bdtData.getAnzahlAerzte().getValue();
            TreeItem<String> item0225 = new TreeItem<>(AnzahlAerzte.getNumber() + " Anzahl Ärzte = " + anzahlAerzte);
            item0010.getChildren().add(item0225);

            TreeItem<String> itemPat = new TreeItem<>("Patientendaten");
            rootItem.getChildren().add(itemPat);
            HashMap<Long, BDTPatient> patients = bdtData.getPatients();
            for (BDTPatient bdtPatient : patients.values()) {
                String patNr = "";
                if (bdtPatient.getPatientNummer() != null)
                    patNr = bdtPatient.getPatientNummer().getValue();
                TreeItem<String> item3000 = new TreeItem<>(Patientennummer_Patientenkennung.getNumber() + " Patientennummer = " + patNr);
                itemPat.getChildren().add(item3000);

                TreeItem<String> item6100 = new TreeItem<>("6100 Patientenstammdaten");
                item3000.getChildren().add(item6100);

                String geburtsDatum = "";
                if (bdtPatient.getGeburtsDatum() != null)
                    geburtsDatum = bdtPatient.getGeburtsDatum().getValue();
                TreeItem<String> item3103 = new TreeItem<>(GeburtsdatumDesPatienten.getNumber() + " Geburtsdatum = " + geburtsDatum);
                item6100.getChildren().add(item3103);

                String geschlecht = "";
                if (bdtPatient.getGeschlecht() != null)
                    geschlecht = bdtPatient.getGeschlecht().getValue();
                TreeItem<String> item3110 = new TreeItem<>(GeschlechtDesPatienten.getNumber() + " Geschlecht = " + geschlecht);
                item6100.getChildren().add(item3110);

                LinkedList<BDTDauerdiagnose> lstDauerDiag = bdtPatient.getDauerDiagnosen();
                if (lstDauerDiag != null) {
                    for (BDTDauerdiagnose bdtDauerdiagnose : lstDauerDiag) {
                        if (bdtDauerdiagnose != null) {
                            String dauerDiagAbDatum = "";
                            dauerDiagAbDatum = bdtDauerdiagnose.getValue();
                            TreeItem<String> item3649 = new TreeItem<>(DauerdiagnosenAbDatum.getNumber() + " Dauerdiagnosen ab Datum = " + dauerDiagAbDatum);
                            item6100.getChildren().add(item3649);
                            LinkedList<BDTValue<String>> lstDiagnosen = bdtDauerdiagnose.getDiagnosen();
                            if (lstDiagnosen != null) {
                                for (BDTValue<String> diagnose : lstDiagnosen) {
                                    if (diagnose != null) {
                                        String diagn = "";
                                        diagn = diagnose.getValue();
                                        TreeItem<String> item3650 = new TreeItem<>(Dauerdiagnosen.getNumber() + " Dauerdiagnose = " + diagn);
                                        item3649.getChildren().add(item3650);
                                    }
                                }
                            }
                        }
                    }
                }
                LinkedList<BDTDauermedikamentation> lstDauerMed = bdtPatient.getDauerMedikamentation();
                if (lstDauerMed != null) {
                    for (BDTDauermedikamentation bdtDauermedikamentation : lstDauerMed) {
                        if (bdtDauermedikamentation != null) {
                            String dauerMedAbDatum = "";
                            dauerMedAbDatum = bdtDauermedikamentation.getValue();
                            TreeItem<String> item3651 = new TreeItem<>(DauermedikamenteAbDatum.getNumber() + " Dauermedikamente ab Datum = " + dauerMedAbDatum);
                            item6100.getChildren().add(item3651);
                            LinkedList<BDTValue<String>> lstMedikamente = bdtDauermedikamentation.getMedikamente();
                            if (lstMedikamente != null) {
                                for (BDTValue<String> medikament : lstMedikamente) {
                                    if (medikament != null) {
                                        String med = "";
                                        med = medikament.getValue();
                                        TreeItem<String> item3652 = new TreeItem<>(Dauermedikamente.getNumber() + " Dauermedikament = " + med);
                                        item3651.getChildren().add(item3652);
                                    }
                                }
                            }
                        }
                    }
                }

                TreeItem<String> itemFall = new TreeItem<>("Behandlungen");
                item3000.getChildren().add(itemFall);

                LinkedList<BDTCase> lstCase = bdtPatient.getBDTCases();
                if (lstCase != null) {
                    for (BDTCase bdtCase : lstCase) {
                        if (bdtCase != null) {
                            TreeItem<String> itemCaseType = null;
                            CaseTypes bdtCaseType = bdtCase.getType();
                            if (bdtCaseType != null) {
                                if (bdtCaseType.equals(CaseTypes.aerztlicheBehandlung)) {
                                    itemCaseType = new TreeItem<>("0101 Ärztliche Behandlungen");
                                    itemFall.getChildren().add(itemCaseType);

                                    String quartalDerAbrechnung = "";
                                    if (bdtCase.getQuartal() != null) {
                                        quartalDerAbrechnung = bdtCase.getQuartal().getValue();
                                        TreeItem<String> item4101 = new TreeItem<>(QuartalDerAbrechnung.getNumber() + " Quartal der Abrechnung = " + quartalDerAbrechnung);
                                        itemCaseType.getChildren().add(item4101);
                                        if (quartalDerAbrechnung.length() == 5) {
                                            String formateQuartal = quartalDerAbrechnung.substring(0, 1) + " / " + quartalDerAbrechnung.substring(1, 5);
                                            itemCaseType.setValue(itemCaseType.getValue() + " (Quartal " + formateQuartal + ")");
                                        }
                                    }

                                    String geschaeftsstelle = "";
                                    if (bdtCase.getGeschaeftsstelle() != null) {
                                        geschaeftsstelle = bdtCase.getGeschaeftsstelle().getValue();
                                        TreeItem<String> item4105 = new TreeItem<>(Geschaeftsstelle.getNumber() + " Geschäftsstelle = " + geschaeftsstelle);
                                        itemCaseType.getChildren().add(item4105);
                                    }

                                    String abrechnungsartSchein = "";
                                    if (bdtCase.getAbrechnungsArt() != null) {
                                        abrechnungsartSchein = bdtCase.getAbrechnungsArt().getValue();
                                        TreeItem<String> item4107 = new TreeItem<>(AbrechnungsartSchein.getNumber() + " Abrechnungsart Schein = " + abrechnungsartSchein);
                                        itemCaseType.getChildren().add(item4107);
                                    }

                                    String gebuehrenordnung = "";
                                    if (bdtCase.getGebuehrenOrdnung() != null) {
                                        gebuehrenordnung = bdtCase.getGebuehrenOrdnung().getValue();
                                        TreeItem<String> item4121 = new TreeItem<>(Gebuehrenordnung.getNumber() + " Gebührenordnung = " + gebuehrenordnung);
                                        itemCaseType.getChildren().add(item4121);
                                    }
                                } else { // Andere Falltypen
                                    itemCaseType = new TreeItem<>(bdtCase.getType().toString());
                                    itemFall.getChildren().add(itemCaseType);
                                }
                            } else {
                                if (!bdtCase.getTreatments().isEmpty()) {
                                    LocalDate date = bdtCase.getTreatments().getFirst().getDate();
                                    int quartal = (date.getMonthValue() - 1) / 3 + 1;
                                    int year = date.getYear();
                                    itemCaseType = new TreeItem<>("6200 Behandlungsdaten (Quartal " + quartal + " /" + year + ")");
                                    itemFall.getChildren().add(itemCaseType);
                                } else
                                    continue;
                            }

                            // Alle Behandlungen
                            LinkedList<BDTTreatment> lstTreatment = bdtCase.getTreatments();
                            if (lstTreatment != null) {
                                for (BDTTreatment bdtTreatment : lstTreatment) {
                                    if (bdtTreatment != null) {

                                        // Datumsknoten
                                        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd.MM.uuuu");
                                        String bdtTreatmentDate = bdtTreatment.getDate().format(formatter);
                                        TreeItem<String> itemDatum = new TreeItem<>("Leistungstag " + bdtTreatmentDate);
                                        itemCaseType.getChildren().add(itemDatum);

                                        // Leistungstage
                                        // GNR
                                        LinkedList<BDTGNR> lstGebuehrennummern = bdtTreatment.getGebuehrennummern();
                                        if (lstGebuehrennummern != null) {
                                            for (BDTGNR bdtGNR : lstGebuehrennummern) {
                                                String gnrText = "";
                                                if (bdtGNR != null) {
                                                    gnrText = bdtGNR.getValue();
                                                    TreeItem<String> item5001 = new TreeItem<>(GNR_GNR_Ident.getNumber() + " GNR = " + gnrText);
                                                    itemDatum.getChildren().add(item5001);
                                                }
                                            }
                                        }

                                        // Abrechnungsdiagnosen + Behandlungsdiagnosen
                                        LinkedList<BDTDiagnose> lstBDTDiagnose = bdtTreatment.getDiagnosen();
                                        if (lstBDTDiagnose != null) {
                                            for (BDTDiagnose bdtDiagnose : lstBDTDiagnose) {
                                                String bdtDiagnoseText = "";
                                                if (bdtDiagnose != null) {
                                                    if (bdtDiagnose.isBillingDiagnosis()) {
                                                        if (bdtDiagnose.isICD()) {
                                                            bdtDiagnoseText = bdtDiagnose.getValue();
                                                            TreeItem<String> item6001 = new TreeItem<>(ICD_Schluessel.getNumber() + " Abrechnungsdiagnose ICD = " + bdtDiagnoseText);
                                                            itemCaseType.getChildren().add(item6001);
                                                        } else {
                                                            bdtDiagnoseText = bdtDiagnose.getValue();
                                                            TreeItem<String> item6000 = new TreeItem<>(Abrechnungsdiagnose.getNumber() + " Abrechnungsdiagnose = " + bdtDiagnoseText);
                                                            itemCaseType.getChildren().add(item6000);
                                                        }
                                                    } else {
                                                        bdtDiagnoseText = bdtDiagnose.getValue();
                                                        TreeItem<String> item6205 = new TreeItem<>(AktuelleDiagnose.getNumber() + " Aktuelle Diagnose = " + bdtDiagnoseText);
                                                        itemDatum.getChildren().add(item6205);
                                                    }

                                                }
                                            }
                                        }

                                        // Medikamente
                                        LinkedList<BDTValue<String>> lstMedikamenteAufRezept = bdtTreatment.getMedikamenteAufRezept();
                                        if (lstMedikamenteAufRezept != null) {
                                            for (BDTValue<String> medikament : lstMedikamenteAufRezept) {
                                                String medikamentVerordnetAufRezept = "";
                                                medikamentVerordnetAufRezept = medikament.getValue();
                                                TreeItem<String> item6210 = new TreeItem<>(
                                                        MedikamentVerordnetAufRezept.getNumber() + " Medikament verordnet auf Rezept = " + medikamentVerordnetAufRezept);
                                                itemDatum.getChildren().add(item6210);
                                            }
                                        }
                                        LinkedList<BDTValue<String>> lstMedikamenteOhneRezept = bdtTreatment.getMedikamenteOhneRezept();
                                        if (lstMedikamenteOhneRezept != null) {
                                            for (BDTValue<String> medikament : lstMedikamenteOhneRezept) {
                                                String medikamenteOhneRezept = "";
                                                medikamenteOhneRezept = medikament.getValue();
                                                TreeItem<String> item6211 = new TreeItem<>(AusserhalbRezeptVerordnetesMedikament.getNumber() + " Medikament ohne Rezept = " + medikamenteOhneRezept);
                                                itemDatum.getChildren().add(item6211);
                                            }
                                        }
                                        LinkedList<BDTValue<String>> lstMedikamentAerztemuster = bdtTreatment.getMedikamentAerztemuster();
                                        if (lstMedikamentAerztemuster != null) {
                                            for (BDTValue<String> medikament : lstMedikamentAerztemuster) {
                                                String medikamentAerztemuster = "";
                                                medikamentAerztemuster = medikament.getValue();
                                                TreeItem<String> item6215 = new TreeItem<>(Aerztemuster.getNumber() + " Medikament Ärztemuster = " + medikamentAerztemuster);
                                                itemDatum.getChildren().add(item6215);
                                            }
                                        }
                                        // Therapie
                                        LinkedList<BDTValue<String>> lstTherapie = bdtTreatment.getTherapie();
                                        if (lstTherapie != null) {
                                            for (BDTValue<String> therapie : lstTherapie) {
                                                String therapieText = "";
                                                therapieText = therapie.getValue();
                                                TreeItem<String> item6260 = new TreeItem<>(Therapie.getNumber() + " Therapie = " + therapieText);
                                                itemDatum.getChildren().add(item6260);
                                            }
                                        }
                                        // PhysikTherapie
                                        LinkedList<BDTValue<String>> lstPhysikTherapie = bdtTreatment.getPhysikTherapie();
                                        if (lstPhysikTherapie != null) {
                                            for (BDTValue<String> therapie : lstPhysikTherapie) {
                                                String therapieText = "";
                                                therapieText = therapie.getValue();
                                                TreeItem<String> item6265 = new TreeItem<>(PhysikalischeTherapie.getNumber() + " Physikalische Therapie = " + therapieText);
                                                itemDatum.getChildren().add(item6265);
                                            }
                                        }
                                        // ÜberweisungInhalt
                                        LinkedList<BDTValue<String>> lstUeberweisungInhalt = bdtTreatment.getUeberweisungInhalt();
                                        if (lstUeberweisungInhalt != null) {
                                            for (BDTValue<String> ueberweisungInhalt : lstUeberweisungInhalt) {
                                                String ueberweisungInhaltText = "";
                                                ueberweisungInhaltText = ueberweisungInhalt.getValue();
                                                TreeItem<String> item6265 = new TreeItem<>(UeberweisungInhalt.getNumber() + " Überweisung Inhalt = " + ueberweisungInhaltText);
                                                itemDatum.getChildren().add(item6265);
                                            }
                                        }
                                        // KrankenhausEinweisungen
                                        BDTKrankenhausEinweisung bdtKrankenhausEinweisung = bdtTreatment.getKrankenhausEinweisungen();
                                        if (bdtKrankenhausEinweisung != null) {
                                            LinkedList<BDTValue<String>> lstEinweisungWegen = bdtKrankenhausEinweisung.getEinweisungWegen();
                                            if (lstEinweisungWegen != null) {
                                                String krankenhausEinweisungenText = "";
                                                for (BDTValue<String> einweisungWegen : lstEinweisungWegen) {
                                                    krankenhausEinweisungenText = einweisungWegen.getValue();
                                                    TreeItem<String> item6291 = new TreeItem<>(
                                                            KrankenhauseinweisungWegen.getNumber() + " Krankenhauseinweisung wegen = " + krankenhausEinweisungenText);
                                                    itemDatum.getChildren().add(item6291);
                                                }
                                            }
                                        }
                                        // Arbeitsunfaehigkeit
                                        BDTArbeitsunfaehigkeit bdtArbeitsunfaehigkeit = bdtTreatment.getArbeitsunfaehigkeit();
                                        if (bdtArbeitsunfaehigkeit != null) {
                                            LinkedList<BDTValue<String>> lstAUWegen = bdtArbeitsunfaehigkeit.getAuWegen();
                                            if (lstAUWegen != null) {
                                                String arbeitsunfaehigkeitText = "";
                                                for (BDTValue<String> auWegen : lstAUWegen) {
                                                    arbeitsunfaehigkeitText = auWegen.getValue();
                                                    TreeItem<String> item6286 = new TreeItem<>(AUWegen.getNumber() + " AUWegen = " + arbeitsunfaehigkeitText);
                                                    itemDatum.getChildren().add(item6286);
                                                }
                                            }
                                        }

                                        // BDTBefundArten
                                        LinkedList<BDTBefundArt> lstBefundArt = bdtTreatment.getBefundArten();
                                        if (lstBefundArt != null) {
                                            for (BDTBefundArt bdtBefundArt : lstBefundArt) {
                                                if (bdtBefundArt != null) {
                                                    String befundart = "";
                                                    befundart = bdtBefundArt.getValue();
                                                    TreeItem<String> item8401 = new TreeItem<>(Befundart.getNumber() + " Befundart = " + befundart);
                                                    itemDatum.getChildren().add(item8401);
                                                    LinkedList<BDTTest> lstTest = bdtBefundArt.getTests();
                                                    for (BDTTest bdtTest : lstTest) {
                                                        if (bdtTest != null) {
                                                            // Test-Ident
                                                            String testident = bdtTest.getValue();
                                                            TreeItem<String> item8410 = new TreeItem<>(Test_Ident.getNumber() + " Test-Ident = " + testident);
                                                            item8401.getChildren().add(item8410);
                                                            // Testbezeichnung
                                                            BDTValue<String> bdtTestBezeichnung = bdtTest.getTestBezeichnung();
                                                            if (bdtTestBezeichnung != null) {
                                                                String testbezeichnung = bdtTestBezeichnung.toString();
                                                                TreeItem<String> item8411 = new TreeItem<>(Testbezeichnung.getNumber() + " Testbezeichnung = " + testbezeichnung);
                                                                item8401.getChildren().add(item8411);
                                                            }
                                                            // Ergebnis_Wert
                                                            BDTValue<String> bdtTestErgebnis = bdtTest.getErgebnis();
                                                            if (bdtTestErgebnis != null) {
                                                                String ergebnisWert = bdtTestErgebnis.toString();
                                                                TreeItem<String> item8420 = new TreeItem<>(Ergebnis_Wert.getNumber() + " Ergebnis_Wert = " + ergebnisWert);
                                                                item8401.getChildren().add(item8420);
                                                            }
                                                            // Einheit
                                                            BDTValue<String> bdtTestEinheit = bdtTest.getEinheit();
                                                            if (bdtTestEinheit != null) {
                                                                String einheit = bdtTestEinheit.toString();
                                                                TreeItem<String> item8421 = new TreeItem<>(Einheit.getNumber() + " Einheit = " + einheit);
                                                                item8401.getChildren().add(item8421);
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                        // remove date of treatments without stored data
                                        if (itemDatum.getChildren().isEmpty()) {
                                            itemDatum.getParent().getChildren().remove(itemDatum);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        radarDataTreeView.setRoot(rootItem);
    }

}